@@ -78,6 +78,7 @@ Exceptions
 
 from __future__ import print_function
 
+
 __docformat__ = "restructuredtext en"
 
 import sys
@@ -87,19 +88,23 @@ import weakref
 import traceback as tb
 from os import listdir, stat
 from os.path import join, isdir, exists
-from typing import Dict, Type, Optional, Union
+from typing import Dict, Type, Optional, Union, Sequence
 from logging import getLogger
 from warnings import warn
+from typing import List, Tuple, Any, Iterable, Callable
+from types import ModuleType
+from typing_extensions import TypedDict
 
 from logilab.common.modutils import modpath_from_file
 from logilab.common.logging_ext import set_log_methods
 from logilab.common.decorators import classproperty
-from logilab.common.deprecation import deprecated
+from logilab.common.deprecation import callable_deprecated
 
 
 # selector base classes and operations ########################################
 
-def objectify_predicate(selector_func):
+
+def objectify_predicate(selector_func: Callable) -> Any:
     """Most of the time, a simple score function is enough to build a selector.
     The :func:`objectify_predicate` decorator turn it into a proper selector
     class::
@@ -112,22 +117,29 @@ def objectify_predicate(selector_func):
             __select__ = View.__select__ & one()
 
     """
-    return type(selector_func.__name__, (Predicate,),
-                {'__doc__': selector_func.__doc__,
-                 '__call__': lambda self, *a, **kw: selector_func(*a, **kw)})
+    return type(
+        selector_func.__name__,
+        (Predicate,),
+        {
+            "__doc__": selector_func.__doc__,
+            "__call__": lambda self, *a, **kw: selector_func(*a, **kw),
+        },
+    )
 
 
 _PREDICATES: Dict[int, Type] = {}
 
-def wrap_predicates(decorator):
+
+def wrap_predicates(decorator: Callable) -> None:
     for predicate in _PREDICATES.values():
-        if not '_decorators' in predicate.__dict__:
+        if "_decorators" not in predicate.__dict__:
             predicate._decorators = set()
         if decorator in predicate._decorators:
             continue
         predicate._decorators.add(decorator)
         predicate.__call__ = decorator(predicate.__call__)
 
+
 class PredicateMetaClass(type):
     def __new__(mcs, *args, **kwargs):
         # use __new__ so subclasses doesn't have to call Predicate.__init__
@@ -158,31 +170,37 @@ class Predicate(object, metaclass=PredicateMetaClass):
         # backward compatibility
         return self.__class__.__name__
 
-    def search_selector(self, selector):
+    def search_selector(self, selector: "Predicate") -> Optional["Predicate"]:
         """search for the given selector, selector instance or tuple of
         selectors in the selectors tree. Return None if not found.
         """
         if self is selector:
             return self
-        if (isinstance(selector, type) or isinstance(selector, tuple)) and \
-               isinstance(self, selector):
+        if (isinstance(selector, type) or isinstance(selector, tuple)) and isinstance(
+            self, selector
+        ):
             return self
         return None
 
     def __str__(self):
         return self.__class__.__name__
 
-    def __and__(self, other):
+    def __and__(self, other: "Predicate") -> "AndPredicate":
         return AndPredicate(self, other)
-    def __rand__(self, other):
+
+    def __rand__(self, other: "Predicate") -> "AndPredicate":
         return AndPredicate(other, self)
-    def __iand__(self, other):
+
+    def __iand__(self, other: "Predicate") -> "AndPredicate":
         return AndPredicate(self, other)
-    def __or__(self, other):
+
+    def __or__(self, other: "Predicate") -> "OrPredicate":
         return OrPredicate(self, other)
-    def __ror__(self, other):
+
+    def __ror__(self, other: "Predicate"):
         return OrPredicate(other, self)
-    def __ior__(self, other):
+
+    def __ior__(self, other: "Predicate") -> "OrPredicate":
         return OrPredicate(self, other)
 
     def __invert__(self):
@@ -191,25 +209,25 @@ class Predicate(object, metaclass=PredicateMetaClass):
     # XXX (function | function) or (function & function) not managed yet
 
     def __call__(self, cls, *args, **kwargs):
-        return NotImplementedError("selector %s must implement its logic "
-                                   "in its __call__ method" % self.__class__)
+        return NotImplementedError(
+            "selector %s must implement its logic " "in its __call__ method" % self.__class__
+        )
 
     def __repr__(self):
-        return u'<Predicate %s at %x>' % (self.__class__.__name__, id(self))
+        return "<Predicate %s at %x>" % (self.__class__.__name__, id(self))
 
 
 class MultiPredicate(Predicate):
     """base class for compound selector classes"""
 
-    def __init__(self, *selectors):
+    def __init__(self, *selectors: Any) -> None:
         self.selectors = self.merge_selectors(selectors)
 
     def __str__(self):
-        return '%s(%s)' % (self.__class__.__name__,
-                           ','.join(str(s) for s in self.selectors))
+        return "%s(%s)" % (self.__class__.__name__, ",".join(str(s) for s in self.selectors))
 
     @classmethod
-    def merge_selectors(cls, selectors):
+    def merge_selectors(cls, selectors: Sequence[Predicate]) -> List[Predicate]:
         """deal with selector instanciation when necessary and merge
         multi-selectors if possible:
 
@@ -231,7 +249,7 @@ class MultiPredicate(Predicate):
                 merged_selectors.append(selector)
         return merged_selectors
 
-    def search_selector(self, selector):
+    def search_selector(self, selector: Predicate) -> Optional[Predicate]:
         """search for the given selector or selector instance (or tuple of
         selectors) in the selectors tree. Return None if not found
         """
@@ -247,7 +265,8 @@ class MultiPredicate(Predicate):
 
 class AndPredicate(MultiPredicate):
     """and-chained selectors"""
-    def __call__(self, cls, *args, **kwargs):
+
+    def __call__(self, cls: Optional[Any], *args: Any, **kwargs: Any) -> int:
         score = 0
         for selector in self.selectors:
             partscore = selector(cls, *args, **kwargs)
@@ -259,15 +278,18 @@ class AndPredicate(MultiPredicate):
 
 class OrPredicate(MultiPredicate):
     """or-chained selectors"""
-    def __call__(self, cls, *args, **kwargs):
+
+    def __call__(self, cls: Optional[Any], *args: Any, **kwargs: Any) -> int:
         for selector in self.selectors:
             partscore = selector(cls, *args, **kwargs)
             if partscore:
                 return partscore
         return 0
 
+
 class NotPredicate(Predicate):
     """negation selector"""
+
     def __init__(self, selector):
         self.selector = selector
 
@@ -276,10 +298,10 @@ class NotPredicate(Predicate):
         return int(not score)
 
     def __str__(self):
-        return 'NOT(%s)' % self.selector
+        return "NOT(%s)" % self.selector
 
 
-class yes(Predicate): # pylint: disable=C0103
+class yes(Predicate):  # pylint: disable=C0103
     """Return the score given as parameter, with a default score of 0.5 so any
     other selector take precedence.
 
@@ -288,7 +310,8 @@ class yes(Predicate): # pylint: disable=C0103
 
     Take care, `yes(0)` could be named 'no'...
     """
-    def __init__(self, score=0.5):
+
+    def __init__(self, score: float = 0.5) -> None:
         self.score = score
 
     def __call__(self, *args, **kwargs):
@@ -297,39 +320,50 @@ class yes(Predicate): # pylint: disable=C0103
 
 # deprecated stuff #############################################################
 
-@deprecated('[lgc 0.59] use Registry.objid class method instead')
-def classid(cls):
-    return '%s.%s' % (cls.__module__, cls.__name__)
 
-@deprecated('[lgc 0.59] use obj_registries function instead')
+@callable_deprecated("[lgc 0.59] use Registry.objid class method instead")
+def classid(cls):
+    return "%s.%s" % (cls.__module__, cls.__name__)
+
+
+@callable_deprecated("[lgc 0.59] use obj_registries function instead")
 def class_registries(cls, registryname):
     return obj_registries(cls, registryname)
 
+
 class RegistryException(Exception):
     """Base class for registry exception."""
 
+
 class RegistryNotFound(RegistryException):
     """Raised when an unknown registry is requested.
 
     This is usually a programming/typo error.
     """
 
+
 class ObjectNotFound(RegistryException):
     """Raised when an unregistered object is requested.
 
     This may be a programming/typo or a misconfiguration error.
     """
 
+
 class NoSelectableObject(RegistryException):
     """Raised when no object is selectable for a given context."""
+
     def __init__(self, args, kwargs, objects):
         self.args = args
         self.kwargs = kwargs
         self.objects = objects
 
     def __str__(self):
-        return ('args: %s, kwargs: %s\ncandidates: %s'
-                % (self.args, self.kwargs.keys(), self.objects))
+        return "args: %s, kwargs: %s\ncandidates: %s" % (
+            self.args,
+            self.kwargs.keys(),
+            self.objects,
+        )
+
 
 class SelectAmbiguity(RegistryException):
     """Raised when several objects compete at selection time with an equal
@@ -338,7 +372,7 @@ class SelectAmbiguity(RegistryException):
     """
 
 
-def _modname_from_path(path, extrapath=None):
+def _modname_from_path(path: str, extrapath: Optional[Any] = None) -> str:
     modpath = modpath_from_file(path, extrapath)
     # omit '__init__' from package's name to avoid loading that module
     # once for each name when it is imported by some other object
@@ -351,26 +385,33 @@ def _modname_from_path(path, extrapath=None):
     #   from package.__init__ import something
     #
     # which seems quite correct.
-    if modpath[-1] == '__init__':
+    if modpath[-1] == "__init__":
         modpath.pop()
-    return '.'.join(modpath)
+    return ".".join(modpath)
 
 
-def _toload_info(path, extrapath, _toload=None):
+def _toload_info(
+    path: List[str], extrapath: Optional[Any], _toload: Optional[Tuple[Dict[str, str], List]] = None
+) -> Tuple[Dict[str, str], List[Tuple[str, str]]]:
     """Return a dictionary of <modname>: <modpath> and an ordered list of
     (file, module name) to load
     """
     if _toload is None:
         assert isinstance(path, list)
         _toload = {}, []
+
     for fileordir in path:
-        if isdir(fileordir) and exists(join(fileordir, '__init__.py')):
+        if isdir(fileordir) and exists(join(fileordir, "__init__.py")):
             subfiles = [join(fileordir, fname) for fname in listdir(fileordir)]
+
             _toload_info(subfiles, extrapath, _toload)
-        elif fileordir[-3:] == '.py':
+
+        elif fileordir[-3:] == ".py":
             modname = _modname_from_path(fileordir, extrapath)
             _toload[0][modname] = fileordir
+
             _toload[1].append((fileordir, modname))
+
     return _toload
 
 
@@ -401,10 +442,10 @@ class RegistrableObject(object):
     __registry__: Optional[str] = None
     __regid__: Optional[str] = None
     __select__: Union[None, str, Predicate] = None
-    __abstract__ = True # see doc snipppets below (in Registry class)
+    __abstract__ = True  # see doc snipppets below (in Registry class)
 
     @classproperty
-    def __registries__(cls):
+    def __registries__(cls) -> Union[Tuple[str], Tuple]:
         if cls.__registry__ is None:
             return ()
         return (cls.__registry__,)
@@ -419,12 +460,13 @@ class RegistrableInstance(RegistrableObject):
         """Add a __module__ attribute telling the module where the instance was
         created, for automatic registration.
         """
-        module = kwargs.pop('__module__', None)
+        module = kwargs.pop("__module__", None)
         obj = super(RegistrableInstance, cls).__new__(cls)
         if module is None:
-            warn('instantiate {0} with '
-                 '__module__=__name__'.format(cls.__name__),
-                 DeprecationWarning)
+            warn(
+                "instantiate {0} with " "__module__=__name__".format(cls.__name__),
+                DeprecationWarning,
+            )
             # XXX subclass must no override __new__
             filepath = tb.extract_stack(limit=2)[0][0]
             obj.__module__ = _modname_from_path(filepath)
@@ -432,10 +474,25 @@ class RegistrableInstance(RegistrableObject):
             obj.__module__ = module
         return obj
 
-    def __init__(self, __module__=None):
+    def __init__(self, __module__: Optional[str] = None) -> None:
         super(RegistrableInstance, self).__init__()
 
 
+SelectBestReport = TypedDict(
+    "SelectBestReport",
+    {
+        "all_objects": List,
+        "end_score": int,
+        "winners": List,
+        "winner": Optional[Any],
+        "self": "Registry",
+        "args": List,
+        "kwargs": Dict,
+        "registry": "Registry",
+    },
+)
+
+
 class Registry(dict):
     """The registry store a set of implementations associated to identifier:
 
@@ -469,10 +526,11 @@ class Registry(dict):
     .. automethod:: possible_objects
     .. automethod:: object_by_id
     """
-    def __init__(self, debugmode):
+
+    def __init__(self, debugmode: bool) -> None:
         super(Registry, self).__init__()
         self.debugmode = debugmode
-        self._select_listeners = []
+        self._select_listeners: List[Callable[[SelectBestReport], None]] = []
 
     def __getitem__(self, name):
         """return the registry (list of implementation objects) associated to
@@ -486,38 +544,39 @@ class Registry(dict):
             raise exc
 
     @classmethod
-    def objid(cls, obj):
+    def objid(cls, obj: Any) -> str:
         """returns a unique identifier for an object stored in the registry"""
-        return '%s.%s' % (obj.__module__, cls.objname(obj))
+        return "%s.%s" % (obj.__module__, cls.objname(obj))
 
     @classmethod
-    def objname(cls, obj):
+    def objname(cls, obj: Any) -> str:
         """returns a readable name for an object stored in the registry"""
-        return getattr(obj, '__name__', id(obj))
+        return getattr(obj, "__name__", id(obj))
 
-    def initialization_completed(self):
+    def initialization_completed(self) -> None:
         """call method __registered__() on registered objects when the callback
         is defined"""
         for objects in self.values():
             for objectcls in objects:
-                registered = getattr(objectcls, '__registered__', None)
+                registered = getattr(objectcls, "__registered__", None)
                 if registered:
                     registered(self)
         if self.debugmode:
             wrap_predicates(_lltrace)
 
-    def register(self, obj, oid=None, clear=False):
+    def register(self, obj: Any, oid: Optional[Any] = None, clear: bool = False) -> None:
         """base method to add an object in the registry"""
-        assert not '__abstract__' in obj.__dict__, obj
+        assert "__abstract__" not in obj.__dict__, obj
         assert obj.__select__, obj
         oid = oid or obj.__regid__
-        assert oid, ('no explicit name supplied to register object %s, '
-                     'which has no __regid__ set' % obj)
+        assert oid, (
+            "no explicit name supplied to register object %s, " "which has no __regid__ set" % obj
+        )
         if clear:
-            objects = self[oid] =  []
+            objects = self[oid] = []
         else:
             objects = self.setdefault(oid, [])
-        assert not obj in objects, 'object %s is already registered' % obj
+        assert obj not in objects, "object %s is already registered" % obj
         objects.append(obj)
 
     def register_and_replace(self, obj, replaced):
@@ -528,15 +587,14 @@ class Registry(dict):
         if not isinstance(replaced, str):
             replaced = self.objid(replaced)
         # prevent from misspelling
-        assert obj is not replaced, 'replacing an object by itself: %s' % obj
+        assert obj is not replaced, "replacing an object by itself: %s" % obj
         registered_objs = self.get(obj.__regid__, ())
         for index, registered in enumerate(registered_objs):
             if self.objid(registered) == replaced:
                 del registered_objs[index]
                 break
         else:
-            self.warning('trying to replace %s that is not registered with %s',
-                         replaced, obj)
+            self.warning("trying to replace %s that is not registered with %s", replaced, obj)
         self.register(obj)
 
     def unregister(self, obj):
@@ -550,8 +608,7 @@ class Registry(dict):
                 self[oid].remove(registered)
                 break
         else:
-            self.warning('can\'t remove %s, no id %s in the registry',
-                         objid, oid)
+            self.warning("can't remove %s, no id %s in the registry", objid, oid)
 
     def all_objects(self):
         """return a list containing all objects in this registry.
@@ -585,9 +642,9 @@ class Registry(dict):
 
         raise :exc:`NoSelectableObject` if no object can be selected
         """
-        obj =  self._select_best(self[__oid], *args, **kwargs)
+        obj = self._select_best(self[__oid], *args, **kwargs)
         if obj is None:
-            raise NoSelectableObject(args, kwargs, self[__oid] )
+            raise NoSelectableObject(args, kwargs, self[__oid])
         return obj
 
     def select_or_none(self, __oid, *args, **kwargs):
@@ -604,7 +661,7 @@ class Registry(dict):
         context
         """
         for objects in self.values():
-            obj = self._select_best(objects,  *args, **kwargs)
+            obj = self._select_best(objects, *args, **kwargs)
             if obj is None:
                 continue
             yield obj
@@ -632,15 +689,16 @@ class Registry(dict):
         once the selection is done and will recieve a dict of the following form::
 
           {"all_objects": [], "end_score": 0, "winners": [], "winner": None or winner,
-          "self": self, "args": args, "kwargs": kwargs, }
+          "self": self, "args": args, "kwargs": kwargs, "registry": self}
         """
 
         if self._select_listeners:
-            select_best_report = {
+            select_best_report: SelectBestReport = {
                 "registry": self,
                 "all_objects": [],
                 "end_score": 0,
                 "winners": [],
+                "winner": None,
                 "self": self,
                 "args": args,
                 "kwargs": kwargs,
@@ -671,7 +729,7 @@ class Registry(dict):
 
         if len(winners) > 1:
             # log in production environement / test, error while debugging
-            msg = 'select ambiguity: %s\n(args: %s, kwargs: %s)'
+            msg = "select ambiguity: %s\n(args: %s, kwargs: %s)"
 
             if self.debugmode:
                 # raise bare exception in debug mode
@@ -692,7 +750,7 @@ class Registry(dict):
     info = warning = error = critical = exception = debug = lambda msg, *a, **kw: None
 
 
-def obj_registries(cls, registryname=None):
+def obj_registries(cls: Any, registryname: Optional[Any] = None) -> Tuple[str]:
     """return a tuple of registry names (see __registries__)"""
     if registryname:
         return (registryname,)
@@ -817,18 +875,18 @@ class RegistryStore(dict):
     key will be the class used when there is no specific class for a name.
     """
 
-    def __init__(self, debugmode=False):
+    def __init__(self, debugmode: bool = False) -> None:
         super(RegistryStore, self).__init__()
         self.debugmode = debugmode
 
-    def reset(self):
+    def reset(self) -> None:
         """clear all registries managed by this store"""
         # don't use self.clear, we want to keep existing subdictionaries
         for subdict in self.values():
             subdict.clear()
-        self._lastmodifs = {}
+        self._lastmodifs: Dict[str, int] = {}
 
-    def __getitem__(self, name):
+    def __getitem__(self, name: str) -> Registry:
         """return the registry (dictionary of class objects) associated to
         this name
         """
@@ -842,9 +900,9 @@ class RegistryStore(dict):
     # methods for explicit (un)registration ###################################
 
     # default class, when no specific class set
-    REGISTRY_FACTORY = {None: Registry}
+    REGISTRY_FACTORY: Dict[Union[None, str], type] = {None: Registry}
 
-    def registry_class(self, regid):
+    def registry_class(self, regid: str) -> type:
         """return existing registry named regid or use factory to create one and
         return it"""
         try:
@@ -852,14 +910,16 @@ class RegistryStore(dict):
         except KeyError:
             return self.REGISTRY_FACTORY[None]
 
-    def setdefault(self, regid):
+    # mypy: Signature of "setdefault" incompatible with supertype "MutableMapping""
+    # I don't know how to overload signatures of method in mypy
+    def setdefault(self, regid: str) -> Registry:  # type: ignore
         try:
             return self[regid]
         except RegistryNotFound:
             self[regid] = self.registry_class(regid)(self.debugmode)
             return self[regid]
 
-    def register_all(self, objects, modname, butclasses=()):
+    def register_all(self, objects: Iterable, modname: str, butclasses: Sequence = ()) -> None:
         """register registrable objects into `objects`.
 
         Registrable objects are properly configured subclasses of
@@ -877,16 +937,23 @@ class RegistryStore(dict):
         :meth:`~logilab.common.registry.RegistryStore.register_and_replace` for
         instance).
         """
-        assert isinstance(modname, str), \
-            'modname expected to be a module name (ie string), got %r' % modname
+        assert isinstance(modname, str), (
+            "modname expected to be a module name (ie string), got %r" % modname
+        )
         for obj in objects:
-            if self.is_registrable(obj) and obj.__module__ == modname and not obj in butclasses:
+            if self.is_registrable(obj) and obj.__module__ == modname and obj not in butclasses:
                 if isinstance(obj, type):
                     self._load_ancestors_then_object(modname, obj, butclasses)
                 else:
                     self.register(obj)
 
-    def register(self, obj, registryname=None, oid=None, clear=False):
+    def register(
+        self,
+        obj: Any,
+        registryname: Optional[Any] = None,
+        oid: Optional[Any] = None,
+        clear: bool = False,
+    ) -> None:
         """register `obj` implementation into `registryname` or
         `obj.__registries__` if not specified, with identifier `oid` or
         `obj.__regid__` if not specified.
@@ -894,12 +961,13 @@ class RegistryStore(dict):
         If `clear` is true, all objects with the same identifier will be
         previously unregistered.
         """
-        assert not obj.__dict__.get('__abstract__'), obj
+        assert not obj.__dict__.get("__abstract__"), obj
         for registryname in obj_registries(obj, registryname):
             registry = self.setdefault(registryname)
             registry.register(obj, oid=oid, clear=clear)
-            self.debug("register %s in %s['%s']",
-                       registry.objname(obj), registryname, oid or obj.__regid__)
+            self.debug(
+                "register %s in %s['%s']", registry.objname(obj), registryname, oid or obj.__regid__
+            )
             self._loadedmods.setdefault(obj.__module__, {})[registry.objid(obj)] = obj
 
     def unregister(self, obj, registryname=None):
@@ -909,8 +977,9 @@ class RegistryStore(dict):
         for registryname in obj_registries(obj, registryname):
             registry = self[registryname]
             registry.unregister(obj)
-            self.debug("unregister %s from %s['%s']",
-                       registry.objname(obj), registryname, obj.__regid__)
+            self.debug(
+                "unregister %s from %s['%s']", registry.objname(obj), registryname, obj.__regid__
+            )
 
     def register_and_replace(self, obj, replaced, registryname=None):
         """register `obj` object into `registryname` or
@@ -921,13 +990,19 @@ class RegistryStore(dict):
         for registryname in obj_registries(obj, registryname):
             registry = self[registryname]
             registry.register_and_replace(obj, replaced)
-            self.debug("register %s in %s['%s'] instead of %s",
-                       registry.objname(obj), registryname, obj.__regid__,
-                       registry.objname(replaced))
+            self.debug(
+                "register %s in %s['%s'] instead of %s",
+                registry.objname(obj),
+                registryname,
+                obj.__regid__,
+                registry.objname(replaced),
+            )
 
     # initialization methods ###################################################
 
-    def init_registration(self, path, extrapath=None):
+    def init_registration(
+        self, path: List[str], extrapath: Optional[Any] = None
+    ) -> List[Tuple[str, str]]:
         """reset registry and walk down path to return list of (path, name)
         file modules to be loaded"""
         # XXX make this private by renaming it to _init_registration ?
@@ -937,11 +1012,11 @@ class RegistryStore(dict):
         # XXX is _loadedmods still necessary ? It seems like it's useful
         #     to avoid loading same module twice, especially with the
         #     _load_ancestors_then_object logic but this needs to be checked
-        self._loadedmods = {}
+        self._loadedmods: Dict[str, Dict[str, type]] = {}
         return filemods
 
-    @deprecated('use register_modnames() instead')
-    def register_objects(self, path, extrapath=None):
+    @callable_deprecated("use register_modnames() instead")
+    def register_objects(self, path: List[str], extrapath: Optional[Any] = None) -> None:
         """register all objects found walking down <path>"""
         # load views from each directory in the instance's path
         # XXX inline init_registration ?
@@ -950,15 +1025,19 @@ class RegistryStore(dict):
             self.load_file(filepath, modname)
         self.initialization_completed()
 
-    def register_modnames(self, modnames):
+    def register_modnames(self, modnames: List[str]) -> None:
         """register all objects found in <modnames>"""
         self.reset()
         self._loadedmods = {}
         self._toloadmods = {}
         toload = []
         for modname in modnames:
-            filepath = pkgutil.find_loader(modname).get_filename()
-            if filepath[-4:] in ('.pyc', '.pyo'):
+            loader = pkgutil.find_loader(modname)
+            assert loader is not None
+            # mypy: "Loader" has no attribute "get_filename"
+            # the selected class has one
+            filepath = loader.get_filename()  # type: ignore
+            if filepath[-4:] in (".pyc", ".pyo"):
                 # The source file *must* exists
                 filepath = filepath[:-1]
             self._toloadmods[modname] = filepath
@@ -967,19 +1046,18 @@ class RegistryStore(dict):
             self.load_file(filepath, modname)
         self.initialization_completed()
 
-    def initialization_completed(self):
+    def initialization_completed(self) -> None:
         """call initialization_completed() on all known registries"""
         for reg in self.values():
             reg.initialization_completed()
 
-    def _mdate(self, filepath):
+    def _mdate(self, filepath: str) -> Optional[int]:
         """ return the modification date of a file path """
         try:
             return stat(filepath)[-2]
         except OSError:
             # this typically happens on emacs backup files (.#foo.py)
-            self.warning('Unable to load %s. It is likely to be a backup file',
-                         filepath)
+            self.warning("Unable to load %s. It is likely to be a backup file", filepath)
             return None
 
     def is_reload_needed(self, path):
@@ -988,30 +1066,29 @@ class RegistryStore(dict):
         """
         lastmodifs = self._lastmodifs
         for fileordir in path:
-            if isdir(fileordir) and exists(join(fileordir, '__init__.py')):
-                if self.is_reload_needed([join(fileordir, fname)
-                                          for fname in listdir(fileordir)]):
+            if isdir(fileordir) and exists(join(fileordir, "__init__.py")):
+                if self.is_reload_needed([join(fileordir, fname) for fname in listdir(fileordir)]):
                     return True
-            elif fileordir[-3:] == '.py':
+            elif fileordir[-3:] == ".py":
                 mdate = self._mdate(fileordir)
                 if mdate is None:
-                    continue # backup file, see _mdate implementation
+                    continue  # backup file, see _mdate implementation
                 elif "flymake" in fileordir:
                     # flymake + pylint in use, don't consider these they will corrupt the registry
                     continue
                 if fileordir not in lastmodifs or lastmodifs[fileordir] < mdate:
-                    self.info('File %s changed since last visit', fileordir)
+                    self.info("File %s changed since last visit", fileordir)
                     return True
         return False
 
-    def load_file(self, filepath, modname):
+    def load_file(self, filepath: str, modname: str) -> None:
         """ load registrable objects (if any) from a python file """
         if modname in self._loadedmods:
             return
         self._loadedmods[modname] = {}
         mdate = self._mdate(filepath)
         if mdate is None:
-            return # backup file, see _mdate implementation
+            return  # backup file, see _mdate implementation
         elif "flymake" in filepath:
             # flymake + pylint in use, don't consider these they will corrupt the registry
             return
@@ -1022,10 +1099,10 @@ class RegistryStore(dict):
         # load the module
         if sys.version_info < (3,) and not isinstance(modname, str):
             modname = str(modname)
-        module = __import__(modname, fromlist=modname.split('.')[:-1])
+        module = __import__(modname, fromlist=modname.split(".")[:-1])
         self.load_module(module)
 
-    def load_module(self, module):
+    def load_module(self, module: ModuleType) -> None:
         """Automatically handle module objects registration.
 
         Instances are registered as soon as they are hashable and have the
@@ -1044,13 +1121,17 @@ class RegistryStore(dict):
         - object class needs to have registries and identifier properly set to a
           non empty string to be registered.
         """
-        self.info('loading %s from %s', module.__name__, module.__file__)
-        if hasattr(module, 'registration_callback'):
-            module.registration_callback(self)
+        self.info("loading %s from %s", module.__name__, module.__file__)
+        if hasattr(module, "registration_callback"):
+            # mypy: Module has no attribute "registration_callback"
+            # we check that before
+            module.registration_callback(self)  # type: ignore
         else:
             self.register_all(vars(module).values(), module.__name__)
 
-    def _load_ancestors_then_object(self, modname, objectcls, butclasses=()):
+    def _load_ancestors_then_object(
+        self, modname: str, objectcls: type, butclasses: Sequence[Any] = ()
+    ) -> None:
         """handle class registration according to rules defined in
         :meth:`load_module`
         """
@@ -1071,7 +1152,7 @@ class RegistryStore(dict):
                 self.load_file(self._toloadmods[objmodname], objmodname)
             return
         # ensure object hasn't been already processed
-        clsid = '%s.%s' % (modname, objectcls.__name__)
+        clsid = "%s.%s" % (modname, objectcls.__name__)
         if clsid in self._loadedmods[modname]:
             return
         self._loadedmods[modname][clsid] = objectcls
@@ -1083,16 +1164,19 @@ class RegistryStore(dict):
             return
         # backward compat
         reg = self.setdefault(obj_registries(objectcls)[0])
-        if reg.objname(objectcls)[0] == '_':
-            warn("[lgc 0.59] object whose name start with '_' won't be "
-                 "skipped anymore at some point, use __abstract__ = True "
-                 "instead (%s)" % objectcls, DeprecationWarning)
+        if reg.objname(objectcls)[0] == "_":
+            warn(
+                "[lgc 0.59] object whose name start with '_' won't be "
+                "skipped anymore at some point, use __abstract__ = True "
+                "instead (%s)" % objectcls,
+                DeprecationWarning,
+            )
             return
         # register, finally
         self.register(objectcls)
 
     @classmethod
-    def is_registrable(cls, obj):
+    def is_registrable(cls, obj: Any) -> bool:
         """ensure `obj` should be registered
 
         as arbitrary stuff may be registered, do a lot of check and warn about
@@ -1101,31 +1185,41 @@ class RegistryStore(dict):
         if isinstance(obj, type):
             if not issubclass(obj, RegistrableObject):
                 # ducktyping backward compat
-                if not (getattr(obj, '__registries__', None)
-                        and getattr(obj, '__regid__', None)
-                        and getattr(obj, '__select__', None)):
+                if not (
+                    getattr(obj, "__registries__", None)
+                    and getattr(obj, "__regid__", None)
+                    and getattr(obj, "__select__", None)
+                ):
                     return False
             elif issubclass(obj, RegistrableInstance):
                 return False
+
         elif not isinstance(obj, RegistrableInstance):
             return False
+
         if not obj.__regid__:
-            return False # no regid
+            return False  # no regid
+
         registries = obj.__registries__
         if not registries:
-            return False # no registries
+            return False  # no registries
+
         selector = obj.__select__
         if not selector:
-            return False # no selector
-        if obj.__dict__.get('__abstract__', False):
+            return False  # no selector
+
+        if obj.__dict__.get("__abstract__", False):
             return False
+
         # then detect potential problems that should be warned
         if not isinstance(registries, (tuple, list)):
-            cls.warning('%s has __registries__ which is not a list or tuple', obj)
+            cls.warning("%s has __registries__ which is not a list or tuple", obj)
             return False
+
         if not callable(selector):
-            cls.warning('%s has not callable __select__', obj)
+            cls.warning("%s has not callable __select__", obj)
             return False
+
         return True
 
     # these are overridden by set_log_methods below
@@ -1134,32 +1228,37 @@ class RegistryStore(dict):
 
 
 # init logging
-set_log_methods(RegistryStore, getLogger('registry.store'))
-set_log_methods(Registry, getLogger('registry'))
+set_log_methods(RegistryStore, getLogger("registry.store"))
+set_log_methods(Registry, getLogger("registry"))
 
 
 # helpers for debugging selectors
 TRACED_OIDS = None
 
+
 def _trace_selector(cls, selector, args, ret):
     vobj = args[0]
-    if TRACED_OIDS == 'all' or vobj.__regid__ in TRACED_OIDS:
-        print('%s -> %s for %s(%s)' % (cls, ret, vobj, vobj.__regid__))
+    if TRACED_OIDS == "all" or vobj.__regid__ in TRACED_OIDS:
+        print("%s -> %s for %s(%s)" % (cls, ret, vobj, vobj.__regid__))
+
 
 def _lltrace(selector):
     """use this decorator on your predicates so they become traceable with
     :class:`traced_selection`
     """
+
     def traced(cls, *args, **kwargs):
         ret = selector(cls, *args, **kwargs)
         if TRACED_OIDS is not None:
             _trace_selector(cls, selector, args, ret)
         return ret
+
     traced.__name__ = selector.__name__
     traced.__doc__ = selector.__doc__
     return traced
 
-class traced_selection(object): # pylint: disable=C0103
+
+class traced_selection(object):  # pylint: disable=C0103
     """
     Typical usage is :
 
@@ -1187,7 +1286,7 @@ class traced_selection(object): # pylint: disable=C0103
     the `logilab.common.registry.Registry.select` method body.
     """
 
-    def __init__(self, traced='all'):
+    def __init__(self, traced="all"):
         self.traced = traced
 
     def __enter__(self):
